@@ -85,6 +85,7 @@ class Puppet::Node::Environment
     @modulepath = modulepath
     @manifest = manifest
     @config_version = config_version
+    @modules_strict_semver = false
   end
 
   # Creates a new Puppet::Node::Environment instance, overriding any of the passed
@@ -213,11 +214,22 @@ class Puppet::Node::Environment
   def validation_errors
     errors = []
     if conflicting_manifest_settings?
-      errors << "The 'disable_per_environment_manifest' setting is true, and the '#{name}' environment has an environment.conf manifest that conflicts with the 'default_manifest' setting."
+      errors << _("The 'disable_per_environment_manifest' setting is true, and the '%{env_name}' environment has an environment.conf manifest that conflicts with the 'default_manifest' setting.") % { env_name: name }
     end
     errors
   end
 
+  # Checks if this environment permits use of rich data types in the catalog
+  # @return [Boolean] `true` if rich data is permitted.
+  # @api private
+  def rich_data?
+    if @rich_data.nil?
+      environment_conf = Puppet.lookup(:environments).get_conf(name)
+      @rich_data = (environment_conf.nil? ? Puppet[:rich_data] : environment_conf.rich_data)
+    end
+    @rich_data
+  end
+
   # Return an environment-specific Puppet setting.
   #
   # @api public
@@ -228,6 +240,22 @@ class Puppet::Node::Environment
     Puppet.settings.value(param, self.name)
   end
 
+  # A SemanticPuppet::VersionRange version >= 1.0.0 will not include versions with pre-release
+  # identifiers unless that is explicitly declared. This may cause backward compatibility
+  # issues when resolving module dependencies and the flag is therefore set to `false` by default.
+  #
+  # @param flag [Boolean] set to true to resolve module dependencies using strict SemVer semantics
+  #
+  def modules_strict_semver=(flag)
+    @modules_strict_semver = flag
+  end
+
+  # @return [Boolean] the current value of the modules_strict_semver flag.
+  # @api public
+  def modules_strict_semver?
+    @modules_strict_semver
+  end
+
   # @api public
   # @return [Puppet::Resource::TypeCollection] The current global TypeCollection
   def known_resource_types
@@ -302,7 +330,7 @@ class Puppet::Node::Environment
 
       @modules = module_references.collect do |reference|
         begin
-          Puppet::Module.new(reference[:name], reference[:path], self)
+          Puppet::Module.new(reference[:name], reference[:path], self, modules_strict_semver?)
         rescue Puppet::Module::Error => e
           Puppet.log_exception(e)
           nil
@@ -344,7 +372,7 @@ class Puppet::Node::Environment
             Puppet::Module.is_module_directory?(name, path)
           end
           modules_by_path[path] = module_names.sort.map do |name|
-            Puppet::Module.new(name, File.join(path, name), self)
+            Puppet::Module.new(name, File.join(path, name), self, modules_strict_semver?)
           end
         end
       else
@@ -519,7 +547,7 @@ class Puppet::Node::Environment
   rescue => detail
     @known_resource_types.parse_failed = true
 
-    msg = "Could not parse for environment #{self}: #{detail}"
+    msg = _("Could not parse for environment %{env}: %{detail}") % { env: self, detail: detail }
     error = Puppet::Error.new(msg)
     error.set_backtrace(detail.backtrace)
     raise error
