@@ -45,6 +45,10 @@ class AbstractWriter
     @packer.flush
   end
 
+  def supports_binary?
+    false
+  end
+
   # Write a value on the underlying stream
   # @api public
   def write(value)
@@ -52,8 +56,8 @@ class AbstractWriter
     case value
     when Integer
       # not tabulated, but integers larger than 64-bit cannot be allowed.
-      raise SerializationError, 'Integer out of bounds' if value > MAX_INTEGER || value < MIN_INTEGER
-    when Numeric, Symbol, Types::PSensitiveType::Sensitive, Extension::NotTabulated, true, false, nil
+      raise SerializationError, _('Integer out of bounds') if value > MAX_INTEGER || value < MIN_INTEGER
+    when Numeric, Symbol, Extension::NotTabulated, true, false, nil
       # not tabulated
     else
       if @tabulate
@@ -134,10 +138,14 @@ class AbstractWriter
       build_payload { |ep| ep.write(o.size) }
     end
 
-    register_type(Extension::OBJECT_START, Extension::ObjectStart) do |o|
+    register_type(Extension::PCORE_OBJECT_START, Extension::PcoreObjectStart) do |o|
       build_payload { |ep| write_tpl_qname(ep, o.type_name); ep.write(o.attribute_count) }
     end
 
+    register_type(Extension::OBJECT_START, Extension::ObjectStart) do |o|
+      build_payload { |ep| ep.write(o.attribute_count) }
+    end
+
     # 0x20 - 0x2f reserved for special extension objects
 
     register_type(Extension::DEFAULT, Extension::Default) do |o|
@@ -148,6 +156,10 @@ class AbstractWriter
       build_payload { |ep| ep.write(o.comment) }
     end
 
+    register_type(Extension::SENSITIVE_START, Extension::SensitiveStart) do |o|
+      build_payload { |ep| }
+    end
+
     # 0x30 - 0x7f reserved for mapping of specific runtime classes
 
     register_type(Extension::REGEXP, Regexp) do |o|
@@ -155,7 +167,7 @@ class AbstractWriter
     end
 
     register_type(Extension::TYPE_REFERENCE, Types::PTypeReferenceType) do |o|
-      build_payload { |ep| write_tpl_qname(ep, o.type_string) }
+      build_payload { |ep| ep.write(o.type_string) }
     end
 
     register_type(Extension::SYMBOL, Symbol) do |o|
@@ -170,23 +182,34 @@ class AbstractWriter
       build_payload { |ep| nsecs = o.nsecs; ep.write(nsecs / 1000000000); ep.write(nsecs % 1000000000) }
     end
 
-    register_type(Extension::VERSION, Semantic::Version) do |o|
+    register_type(Extension::VERSION, SemanticPuppet::Version) do |o|
       build_payload { |ep| ep.write(o.to_s) }
     end
 
-    register_type(Extension::VERSION_RANGE, Semantic::VersionRange) do |o|
+    register_type(Extension::VERSION_RANGE, SemanticPuppet::VersionRange) do |o|
       build_payload { |ep| ep.write(o.to_s) }
     end
 
-    register_type(Extension::SENSITIVE, Types::PSensitiveType::Sensitive) do |o|
-      build_payload { |ep| ep.write(o.unwrap) }
+    if supports_binary?
+      register_type(Extension::BINARY, Types::PBinaryType::Binary) do |o|
+        # The Ruby MessagePack implementation has special treatment for "ASCII-8BIT" strings. They
+        # are written as binary data.
+        build_payload { |ep| ep.write(o.binary_buffer) }
+      end
+    else
+      register_type(Extension::BASE64, Types::PBinaryType::Binary) do |o|
+        build_payload { |ep| ep.write(o.to_s) }
+      end
     end
+  end
 
-    register_type(Extension::BINARY, Types::PBinaryType::Binary) do |o|
-      build_payload { |ep| ep.write(o.to_s) }
-    end
+  def to_s
+    "#{self.class.name}"
+  end
+
+  def inspect
+    to_s
   end
 end
 end
 end
-
