@@ -10,11 +10,11 @@ class ObjectReader
   include InstanceReader
 
   def read(impl_class, value_count, deserializer)
-    type = impl_class._ptype
+    type = impl_class._pcore_type
     (names, types, required_count) = type.parameter_info(impl_class)
     max = names.size
     unless value_count >= required_count && value_count <= max
-      raise Serialization::SerializationError, "Feature count mismatch for #{impl_class.name}. Expected #{required_count} - #{max}, actual #{value_count}"
+      raise Serialization::SerializationError, _("Feature count mismatch for %{value0}. Expected %{required_count} - %{max}, actual %{value_count}") % { value0: impl_class.name, required_count: required_count, max: max, value_count: value_count }
     end
     # Deserializer must know about this instance before we read its attributes
     val = deserializer.remember(impl_class.allocate)
@@ -25,7 +25,7 @@ class ObjectReader
         Types::TypeAsserter.assert_instance_of(nil, ptype, arg) { "#{type.name}[#{names[index]}]" } unless arg == :default
       else
         attr = type[names[index]]
-        raise Serialization::SerializationError, "Missing default value for #{type.name}[#{names[index]}]" unless attr.value?
+        raise Serialization::SerializationError, _("Missing default value for %{type_name}[%{name}]") % { type_name: type.name, name: names[index] } unless attr.value?
         args << attr.value
       end
     end
@@ -43,16 +43,16 @@ class ObjectWriter
 
   def write(type, value, serializer)
     impl_class = value.class
-    (names, types, required_count) = type.parameter_info(impl_class, true)
+    (names, types, required_count) = type.parameter_info(impl_class)
     args = names.map { |name| value.send(name) }
 
     # Pop optional arguments that are default
     while args.size > required_count
-      break unless args.last == type[names[args.size-1]].value
+      break unless type[names[args.size-1]].default_value?(args.last)
       args.pop
     end
 
-    if type.name.start_with?('Pcore::')
+    if type.name.start_with?('Pcore::') || serializer.type_by_reference?
       serializer.push_written(value)
       serializer.start_pcore_object(type.name, args.size)
     else
