@@ -85,13 +85,17 @@ class TypeParser
   end
 
   # @api private
-  def interpret_QualifiedName(o, context)
-    o.value
+  def interpret_HeredocExpression(o, context)
+    interpret_any(o.text_expr, context)
+  end
+
+  def interpret_SubLocatedExpression(o, context)
+    interpret_any(o.expr, context)
   end
 
   # @api private
-  def interpret_QualifiedReference(o, context)
-    o.cased_value
+  def interpret_QualifiedName(o, context)
+    o.value
   end
 
   # @api private
@@ -338,6 +342,7 @@ class TypeParser
     when 'enum'
       # 1..m parameters being strings
       raise_invalid_parameters_error('Enum', '1 or more', parameters.size) unless parameters.size >= 1
+      parameters.each { |p|  raise Puppet::ParseError, 'Enum parameters must be identifiers or strings' unless p.is_a?(String) }
       TypeFactory.enum(*parameters)
 
     when 'pattern'
@@ -348,6 +353,7 @@ class TypeParser
     when 'variant'
       # 1..m parameters being strings or regular expressions
       raise_invalid_parameters_error('Variant', '1 or more', parameters.size) unless parameters.size >= 1
+      parameters.each { |p| assert_type(ast, p) }
       TypeFactory.variant(*parameters)
 
     when 'tuple'
